# =====================================================================
# (C) 2019-2021 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from . import SSH_Connection
from . import VIOS_Handler
import re
import time
import six
from six.moves import configparser
import os
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("HMCHandlerModule")
VIOS_CMD = CONF.get('vios_profile', 'VIOS_CMD_STR')


class HMChandler:

    def __init__(self, ):
        self.viosHandler = VIOS_Handler.VIOShandler()

    # For creating SSH connection
    def createSSHconnection(self, ipAddress, port, username, password):
        SSHConnectionObj = SSH_Connection.SSHConnection()
        self.sshClient = SSHConnectionObj.createSSHconnection(
            ipAddress, port, username, password)
        return self.sshClient

    # For executing SSH Commands
    def executeCommand(self, command):
        log.info("executeCommand : "+command)
        SSHConnectionObj = SSH_Connection.SSHConnection()
        stdout = SSHConnectionObj.executeCommand(self.sshClient, command)
        return stdout

    # For executing Non Blocking SSH Commands for VIOS Installation
    def executeViosInstallation(self, command):
        log.info("executeViosInstallation: "+command)
        SSHConnectionObj = SSH_Connection.SSHConnection()
        stdout = SSHConnectionObj.executeViosInstallation(
            self.sshClient, command)
        return stdout

    # For executing Non Blocking SSH Commands
    def executeCommandNonBlocking(self, command):
        log.info("executeCommandNonBlocking: "+command)
        SSHConnectionObj = SSH_Connection.SSHConnection()
        stdout = SSHConnectionObj.executeCommandNonBlocking(
            self.sshClient, command)
        return stdout

    # For closing SSH connection
    def closeSSHconnection(self):
        SSHConnectionObj = SSH_Connection.SSHConnection()
        stdout = SSHConnectionObj.closeSSHconnection(self.sshClient)
        log.info("SSH Connection closed")
        return stdout

    # For parsing comma separated data
    def parseCommaSeparatedData(self, data):
        if data:
            regex = re.compile(
                r'''
                (?P<key>\w+)=
                (?P<quote>["']?)
                (?P<value>.*?)
                (?P=quote)
                ($|,)
                ''',
                re.VERBOSE)
            data = {match.group('key'): match.group(
                'value') for match in regex.finditer(data)}
        return data

    def formatHmcOutput(self, line):
        line = line.strip()
        line = line.strip("\n")
        line = line.replace("\r", "")
        return line

    # HMC Commands
    # listing all LPAR within a managed system
    # input paramters : frame name
    # Output : List of VMs
    def listLpar(self, frameName):
        command = 'lssyscfg -m '+frameName+' -r lpar -F name,lpar_id,state'
        response = self.executeCommand(command)
        lparList = []
        if response['code'] == 0:
            for line in response['stdout']:
                line = self.formatHmcOutput(line)
                res = line.split(",")
                if len(res) > 2:
                    lpar_info = {}
                    lpar_info["lpar_name"] = res[0]
                    lpar_info["lpar_id"] = res[1]
                    lpar_info["state"] = res[2]
                    lparList.append(lpar_info)
        else:
            for line in response['stdout']:
                print(line)
            exit(1)
        return lparList

    # listing all managed system within a HMC
    # input parameters : None
    # Output : List of managed systems
    def sys_list(self, ):
        command = 'lssyscfg -r sys -F name'
        response = self.executeCommand(command)
        system_list = []
        if response['code'] == 0:
            for line in response['stdout']:
                line = self.formatHmcOutput(line)
                res = line.split(",")
                if len(res) > 0:
                    system_list.append(res[0])
        else:
            for line in response['stdout']:
                print(line)
            exit(1)
        return system_list

    # Hardware details for CSV templates
    def getNetworkInfo(self, hmcIp):
        log.info("Inside getNetworkInfo method start")
        command = 'lshmc -n | grep '+hmcIp
        response = self.executeCommand(command)
        data = {}
        for line in response['stdout']:
            line = self.formatHmcOutput(line)
            res = line.replace("\r", "")
            data = self.parseCommaSeparatedData(res)
        return data

    # For fetching all IO SLOTS withing managed system
    def getIoSlots(self, frameName):
        log.info("Inside getIoSlots method start")
        command = 'lshwres -r io -m '+frameName+' --rsubtype slot'
        response = self.executeCommand(command)
        data = {}
        allSlotList = []
        if response['code'] == 0:
            for line in response['stdout']:
                res = self.formatHmcOutput(line)
                data = self.parseCommaSeparatedData(res)
                allSlotList.append(data)
        else:
            for line in response['stdout']:
                print(line)
            exit(1)
        return allSlotList

    # For fetching all SRIOV Adapters
    def getSRIOVadapters(self, frameName):
        log.info("Inside getSRIOVadapters method start")
        command = 'lshwres -r sriov -m '+frameName+' --rsubtype adapter'
        response = self.executeCommand(command)
        data = {}
        allSlotList = []
        if response['code'] == 0:
            for line in response['stdout']:
                res = self.formatHmcOutput(line)
                data = self.parseCommaSeparatedData(res)
                allSlotList.append(data)
        else:
            for line in response['stdout']:
                print(line)
            exit(1)
        return allSlotList

    def get_sriov_ports(self, frameName, drcName, portType):
        log.info("Inside get_sriov_ports method start")
        command = 'lshwres -r sriov -m '+str(
            frameName)+' --rsubtype physport --level '+str(
            portType)+' | grep -w '+drcName
        response = self.executeCommand(command)
        data = {}
        portList = []
        if response['code'] == 0:
            for line in response['stdout']:
                res = self.formatHmcOutput(line)
                data = self.parseCommaSeparatedData(res)
                data['port_type'] = portType
                portList.append(data)
        else:
            for line in response['stdout']:
                print(line)
        return portList

    def get_sriov_eth_ports(self, frameName, drcName):
        log.info("Inside get_sriov_eth_ports method start")
        port_list = self.get_sriov_ports(frameName, drcName, 'eth')
        return port_list

    def get_sriov_ethc_ports(self, frameName, drcName):
        log.info("Inside get_sriov_ethc_ports method start")
        port_list = self.get_sriov_ports(frameName, drcName, 'ethc')
        return port_list

    def get_sriov_roce_ports(self, frameName, drcName):
        log.info("Inside get_sriov_roce_ports method start")
        port_list = self.get_sriov_ports(frameName, drcName, 'roce')
        return port_list

    def get_sriov_adapter_ports(self, frameName, drcName):
        log.info("Inside get_sriov_adapter_ports method start")
        port_list = []
        port_list_eth = self.get_sriov_eth_ports(frameName, drcName)
        port_list_ethc = self.get_sriov_ethc_ports(frameName, drcName)
        port_list_roce = self.get_sriov_roce_ports(frameName, drcName)
        port_list.extend(port_list_eth)
        port_list.extend(port_list_ethc)
        port_list.extend(port_list_roce)
        return port_list

    def get_sriov_ports_filtered(self, frameName, drcName, portType):
        cap = "min_eth_capacity_granularity"
        command = 'lshwres -r sriov -m '+str(
            frameName)+' --rsubtype physport --level '+str(
            portType)+' -F adapter_id,phys_port_id,'+str(
            cap)+',phys_port_loc | grep -w '+drcName
        response = self.executeCommand(command)
        port_info = False
        if response['code'] == 0:
            for line in response["stdout"]:
                line = self.formatHmcOutput(line)
                lineRes = line.split(",")
                port_info = {}
                port_info["adapter_id"] = lineRes[0]
                port_info["phys_port_id"] = lineRes[1]
                port_info["min_eth_capacity_granularity"] = lineRes[2]
                port_info["phys_port_loc"] = lineRes[3]
        return port_info

    def assign_sriov_port(self, frameName, viosName, adapterId, pyhsPortId):
        command = 'chhwres -r sriov --rsubtype logport -m '+str(
            frameName)+' -p '+str(viosName)+' -o a -a "adapter_id='+str(
            adapterId)+',phys_port_id='+str(
            pyhsPortId)+',logical_port_type=eth"'
        response = self.executeCommand(command)
        return response

    # Verifying for valid path on HMC
    def checkISOlocation(self, path):
        command = 'ls '+str(path)
        response = self.executeCommand(command)
        if response['code'] == 0:
            return True
        else:
            return False

    # Fetch DRC Index using DRC Name
    # DRC Index used for mapping IO Slots to profile
    def getDrcIndexUsingDrcName(self, frameName, drc_name):
        command = 'lshwres -r io -m '+str(
            frameName)+' --rsubtype slot | grep -w '+drc_name
        response = self.executeCommand(command)
        data = {}
        allSlotList = []
        if response['code'] == 0:
            for line in response['stdout']:
                res = self.formatHmcOutput(line)
                data = self.parseCommaSeparatedData(res)
        return data

    # Ping test
    # -c used for ping count
    # After one ping count exit from ping test
    def pingTest(self, ipAddress):
        command = 'ping -c 1 '+ipAddress
        response = self.executeCommand(command)
        return response

    # Applying profile to VM
    def applyProfile(self, frameName, lparName, profileName):
        command = 'chsyscfg -r lpar -m '+str(frameName)+' -o apply -p '+str(
            lparName)+' -n '+profileName
        response = self.executeCommand(command)
        return response

    # Applying service partiion to Machine
    def apply_service_partition(self, frameName, lparId):
        command = 'chsyscfg -r sys -m '+str(
            frameName)+' -i "service_lpar_id='+str(lparId)+'"'
        response = self.executeCommand(command)
        return response

    # Create profile for VM
    # Formatting cli parameters for adapters in specific format
    def createProfile(self, frameName, lparName, profileName, optParams):
        params = ""
        for key, value in optParams.items():
            if 'virtual_eth_adapters' == str(
                    key) or 'virtual_scsi_adapters' == str(
                    key) or 'virtual_fc_adapters' == key or 'io_slots' == str(
                    key) or 'sriov_eth_logical_ports' == key:
                params = params + ',\\"'+str(key)+'='+str(value)+'\\"'
            else:
                params = params + ','+str(key)+'='+str(value)

        command = 'mksyscfg -r lpar -m '+str(frameName)+' -i name='+str(
            lparName)+', profile_name='+profileName+params
        response = self.executeCommand(command)
        return response

    # Listing all VMs within a managed system
    # Filtering VM using VM Name
    def listLparUsingLparName(self, frameName, lparName):
        command = 'lssyscfg -m '+str(
            frameName)+' -r lpar -F name,lpar_id,state | grep -w '+lparName
        response = self.executeCommand(command)
        lparList = []
        if response['code'] == 0:
            for line in response['stdout']:
                line = self.formatHmcOutput(line)
                res = line.split(",")
                if len(res) > 2:
                    lpar_info = {}
                    lpar_info["lpar_name"] = res[0]
                    lpar_info["lpar_id"] = res[1]
                    lpar_info["state"] = res[2]
                    lparList.append(lpar_info)
        else:
            for line in response['stdout']:
                print(line)
        return lparList

    # Immediate shutdown of LPAR
    def lparChangeStateImmedShutdown(self, frameName, lparName):
        command = 'chsysstate -r lpar -o shutdown --immed -n '+str(
            lparName)+' -m '+frameName
        response = self.executeCommand(command)
        return response

    # Activate VM with current configuration
    def lparActivateProfileCurrentConfiguration(self, frameName, lparName):
        log.info("Inside lparActivateProfileCurrentConfiguration method")
        command = 'chsysstate -r lpar -o on -n '+str(
            lparName)+' -m '+frameName
        response = self.executeCommand(command)
        return response

    # Close VM terminal
    def closeLparTerminal(self, frameName, lparName):
        command = 'rmvterm -m '+frameName+' -p '+lparName
        response = self.executeCommand(command)
        return response

    # Delete LPAR profile
    def deleteProfile(self, frameName, lparName):
        res = self.closeLparTerminal(frameName, lparName)
        if res['code'] == 0:
            command = 'rmsyscfg -r lpar -m '+frameName+' -n '+lparName
            response = self.executeCommand(command)
            return response
        else:
            return res

    # Checking VM status.
    # If state is not "Not Activated", shutting down VM then deleting it
    # Else directly deleting
    def shutdownAndDeleteProfile(self, frameName, lparName):
        res = self.listLparUsingLparName(frameName, lparName)
        for info in res:
            if info['state'] != "Not Activated":
                self.lparChangeStateImmedShutdown(frameName, lparName)
                count = 0
                while count <= 10:
                    res = self.listLparUsingLparName(frameName, lparName)
                    for info in res:
                        if info['state'] == "Not Activated":
                            res = self.deleteProfile(frameName, lparName)
                            return res
                        else:
                            count = count+1
                            time.sleep(10)
                else:
                    return {"code": 1, "stdout": "Request timeout"}
            else:
                res = self.deleteProfile(frameName, lparName)
                return res

    # lpar_netboot for ping test
    # Checking ping success for Physical adapters
    # In case of ping success, returning ping info with MAC address
    # MAC Address is being used for VIOS Install in installios command
    # Printing console logs for VM different state
    # and ping states of adapters
    def lparNetbootPingTestVios(self, info):
        frameName = info['frameName']
        viosName = info['viosName']
        profileName = info['profileName']
        serverIp = info['serverIp']
        viosIp = info['viosIp']
        subnetMask = info['subnetMask']
        gatewayIp = info['gatewayIp']
        GOLD_BUILD = frameName+viosName
        GOLD_BUILD = GOLD_BUILD.replace("-", "").replace("_", "")
        self.clearOlderResOfInstallios(GOLD_BUILD)
        # Parameters details
        # -i : Force immediate shutdown
        # -A : Return all adapters of the given type
        # -M : Discovery ethernet adapter mac address and location code
        # -f : Force close virtual terminal session
        # -T : Enable or disble spanning tree
        # -D : Perform ping test, use adapter that successfully pings server
        # -K : Subnetmask IP address
        # -G : Gateway IP address
        # -C : Client IP address (VIOS/VM IP)
        # -S : Server IP (BootP)
        # -d : Network adapter duplex
        # -s : Network adapter speed
        # -t : Specifies network type ent
        # -n : Do not boot partition

        command = "lpar_netboot -i -A -M -f -T off -D -K "+str(
            subnetMask)+" -G "+gatewayIp+" -C "+viosIp+" -S "+str(
            serverIp)+" -d auto -s auto -t ent -n "+viosName+" "+str(
            profileName)+" "+frameName
        try:
            log.info(command)
            stdin, stdout, stderr = self.sshClient.exec_command(command)
            for x in stdout:
                log.info(str(x))
                print(x)
                PING_PASS = CONF.get('vios_profile', 'VIOS_PING_SUCCESS_STR')
                PING_FAIL = CONF.get('vios_profile', 'VIOS_PING_FAIL_STR')
                if PING_PASS in x and PING_FAIL not in x:
                    return self.formatResponse(0, x, stderr)
            for x in stderr:
                log.info(str(x))
                print(x)
            return self.formatResponse(1, stdout, stderr)

        except Exception as e:
            print(str(e))
            return self.formatResponse(3, str(e), str(e))

    # Checking LPAR status, if not in "Not Activated" state
    # It will keep checking upto 15 * 10 = 150 sec
    def checkLparShutDownState(self, frameName, lparName):
        res = self.listLparUsingLparName(frameName, lparName)
        for info in res:
            if info['state'] != "Not Activated":
                self.lparChangeStateImmedShutdown(frameName, lparName)
                count = 0
                while count <= 10:
                    res = self.listLparUsingLparName(frameName, lparName)
                    for info in res:
                        if info['state'] == "Not Activated":
                            return True
                        else:
                            count = count+1
                            time.sleep(15)

    # Clearing older resources of installios commands
    def clearOlderResOfInstallios(self, GOLD_BUILD):
        # command = "installios -u"
        # response = self.executeCommand(command)
        command = "installios -F -e -R "+GOLD_BUILD
        response = self.executeCommand(command)

    # Formatting response code, output and error
    def formatResponse(self, code, stdout, stderr):
        return {"code": code,
                "stdout": stdout,
                "stderr": stderr}

    # Formatting installios response output for console printing
    def formatInstalliosRes(self, data):
        val = data.split('info=')
        if len(val) > 1:
            return val[1]
        else:
            return val[0]

    # VIOS Install function
    # Printing installation status on console
    # Filtering output for BOS Install % status
    def installios(self, info):
        frameName = info['frameName']
        viosName = info['viosName']
        viosIsoPath = info['viosIsoPath']
        profileName = info['profileName']
        gatewayIp = info['gatewayIp']
        viosIp = info['viosIp']
        subnetMask = info['subnetMask']
        hmcNetworkEth = info['hmcNetworkEth']
        pingTest = info['pingTest']
        GOLD_BUILD = frameName+viosName
        GOLD_BUILD = GOLD_BUILD.replace("-", "").replace("_", "")
        print("Proceeding for VIOS Installation for "+str(
            viosName)+", Please wait ...")
        installios_dest_dir = CONF.get('vios_profile', 'INSTALLIOS_DEST_DIR')
        time.sleep(30)
        self.checkLparShutDownState(frameName, viosName)
        self.clearOlderResOfInstallios(GOLD_BUILD)

        VIOS_PHY_NWT_BOOT_DEVICE_FAILED = CONF.get(
            'hmc_error', 'VIOS_PHY_NWT_BOOT_DEVICE_FAILED')
        VIOS_PROMPT_USER_INPUT = CONF.get(
            'hmc_error', 'VIOS_PROMPT_USER_INPUT')
        VIOS_BOOT_INVALID = CONF.get('hmc_error', 'VIOS_BOOT_INVALID')

        # Parameters details
        # -d : VIOS ISO path location on HMC
        # -m : MAC address of IO Adapter
        # -i : Client IP Address(VIOS IP)
        # -g : Gateway IP Address
        # -S : Subnet Mask IP Address
        # -s : Machine Name
        # -p : VIOS Name
        # -r : Profile Name
        # -A : Network Interface
        # -R : Label Name for installios
        # -C : Specifies to save the label name

        command = "installios -d "+viosIsoPath+" -i "+str(
            viosIp)+" -g "+gatewayIp+" -S "+subnetMask+" -s "+str(
            frameName)+" -p "+viosName+" -r "+profileName+" -A "+str(
            hmcNetworkEth)+" -R "+GOLD_BUILD+" -C -T "+installios_dest_dir
        if pingTest:
            macAddress = info['macAddress']
            command = command+" -m "+macAddress
        log.info(command)
        try:
            stdin, stdout, stderr = self.sshClient.exec_command(command)
            old_code = ""
            install_success = False
            for x in stdout:
                print("x in stdout: "+str(x))
                if 'nimol_config' in x or 'nimol_install' in str(
                        x) or 'R,success' in x:
                    log.info(str(x))
                if 'code=' in x:
                    code = x.split("code=")[1]
                    new_code = code.split(":")[0].strip()
                    if old_code != new_code:
                        print(self.formatInstalliosRes(x))
                        log.info(str(x))
                        old_code = new_code
                elif 'installios' in x:
                    log.info(str(x))
                if 'bos install' in x.lower() or '# ' in x:
                    print(self.formatInstalliosRes(x))
                    log.info(str(x))
                if 'unsuccessful' in x:
                    log.info(str(x))
                    print("Ping failed...")
                    return self.formatResponse(1, x, stderr)
                if 'Finished' in x:
                    log.info(str(x))
                    print(MSGCATALOG.VIOS_INSTALL_MESSAGE.format(viosName))
                    print(MSGCATALOG.WAITING_MSG_VIOS_INSTALL)
                if 'bos install 100% complete' in x.lower():
                    log.info(str(x))
                    time.sleep(10)
                    try:
                        self.clearOlderResOfInstallios(GOLD_BUILD)
                    except Exception as e:
                        log.error("Unable to clean install resource: "+str(e))
                    return self.formatResponse(0, x, stderr)
                if 'BOS Install:' in x and 'fail' in x:
                    log.info(str(x))
                    print(self.formatInstalliosRes(x))
                    return self.formatResponse(2, stdout, stderr)
                if 'R,success' in x:
                    install_success = True
                if VIOS_BOOT_INVALID in x:
                    print("Installation failed, error code : 0c48")
                    print("For more details, mkvterm to VIOS.")
                    return self.formatResponse(2, stdout, stderr)
                if VIOS_PROMPT_USER_INPUT in x:
                    print("Installation failed, error code : CA00E1F7")
                    return self.formatResponse(2, stdout, stderr)
                if VIOS_PHY_NWT_BOOT_DEVICE_FAILED in x:
                    print("Installation failed, error code : "+str(
                        VIOS_PHY_NWT_BOOT_DEVICE_FAILED))
                    print(MSGCATALOG.VIOS_PHY_NWT_BOOT_DEVICE_ERROR)
                    return self.formatResponse(2, stdout, stderr)

            if install_success:
                is_rmc_active = self.get_rmc_installios(frameName, viosName)
                if is_rmc_active:
                    log.info("RMC Active, retuning to main function")
                    return self.formatResponse(0, x, stderr)

            print("Error Occurred")
            return self.formatResponse(2, stdout, stderr)
        except Exception as e:
            print(str(e))
            return self.formatResponse(3, str(e), str(e))

    def get_rmc_installios(self, frameName, viosName):
        log.info("inside get_rmc_installios method")
        for x in range(50):
            rmc_active = False
            res = self.getViosRmcState(frameName, viosName)
            log.info("RMC State for "+viosName+" : "+res['rmc_state'])
            if res['code'] == 200:
                if res['rmc_state'] == "active":
                    rmc_active = True
                    return rmc_active
            time.sleep(20)
        log.error("Request timeout, RMC is not active")
        return False

    # Fetching VIOS RMC State
    # Filtering VIOS using VIOS name
    def getViosRmcState(self, frameName, viosName):
        command = "lssyscfg -m "+str(
            frameName)+" -r lpar -F name,state,rmc_state | grep -w "+viosName
        response = self.executeCommand(command)
        if response["code"] == 0:
            vios_info = {}
            for line in response["stdout"]:
                line = self.formatHmcOutput(line)
                lineRes = line.split(",")
                vios_info["vios_name"] = lineRes[0]
                vios_info["status"] = lineRes[1]
                vios_info["rmc_state"] = lineRes[2]
                vios_info["code"] = 200
            return vios_info
        else:
            return {"code": 201, "error": "Unable to find vios status"}

    def invokeShell(self, channel, cmd):
        channel.send(cmd+'\r')
        while not channel.recv_ready():
            time.sleep(3)
        out = channel.recv(9999)
        data = out.decode("ascii")
        log.info(str(data))
        return data

    def formatShellRes(self, data, type):
        if type == "frame":
            res = data.split("Managed Systems:")
        elif type == "vios":
            res = data.split("Managed System:")
        vals = res[1].strip()
        vals = vals.strip()
        vals = vals.split("\r")
        return vals

    def getFrameNum(self, data, frameName):
        vals = self.formatShellRes(data, 'frame')
        frameNum = False
        for x in vals:
            info = x.strip().split(")")
            if len(info) > 1:
                if frameName == info[1].strip():
                    frameNum = info[0]
        return frameNum

    def getViosNum(self, data, viosName):
        vals = self.formatShellRes(data, 'vios')
        viosNum = False
        for x in vals:
            info = x.strip().split(")")
            if len(info) > 1:
                result = info[1].strip().split(" ")
                if viosName == result[0].strip():
                    viosNum = info[0]
        return viosNum

    # Close VIOS Vterm
    def rmvterm(self, frameName, viosName):
        log.info("Close VIOS vterm")
        command = "rmvterm -m "+frameName+" -p "+viosName
        response = self.executeCommand(command)
        return response

    # Login to vtmenu
    # Search for Managed System
    # Search for VIOS
    # Login to VIOS
    # Set new password and confirm password
    # Deploy rules
    # Restart VIOS
    def configureVIOS(self, frameName, viosName, login, password):
        # close vios vterm
        self.rmvterm(frameName, viosName)
        log.info("Configuring VIOS")
        channel = self.sshClient.invoke_shell()
        out = channel.recv(9999)
        data = self.invokeShell(channel, 'vtmenu')
        sys_list = self.sys_list()
        log.info("Total managed system found :"+str(len(sys_list)))
        if len(sys_list) > 1:
            frameNum = self.getFrameNum(data, frameName)
            log.info("Frame found in vtmenu result :"+str(frameNum))
            data = self.invokeShell(channel, str(frameNum))
        viosNum = self.getViosNum(data, viosName)
        log.info("VIOS number found as :"+str(viosNum))
        res = self.invokeShell(channel, str(viosNum))
        # Setting VIOS login
        res = self.invokeShell(channel, login)
        # Setting VIOS new Password
        res = self.invokeShell(channel, password)
        # Setting VIOS confirm Password
        res = self.invokeShell(channel, password)
        print(MSGCATALOG.VIOS_SET_PASSWORD)
        log.info("VIOS confirm password done")
        res = self.invokeShell(channel, "")
        # Accepting license
        res = self.invokeShell(channel, 'license -accept')
        time.sleep(3)
        print(MSGCATALOG.VIOS_ACCEPT_LICENSE)
        log.info("VIOS license accept done")
        # Rule deploy
        res = self.invokeShell(channel, 'rules -o deploy -d')
        print(MSGCATALOG.VIOS_DEPLOY_RULE)
        log.info("VIOS rules deploy done")
        # Restart VIOS
        res = self.invokeShell(channel, 'shutdown -restart')
        time.sleep(3)
        log.info("VIOS restarting")
        res = self.invokeShell(channel, 'y')
        time.sleep(10)
        print(MSGCATALOG.VIOS_RESTART)
        log.info("VIOS restarting confirmed")
        time.sleep(20)
        return True

    # SEA COMMANDS
    # Get virtual Adapters
    def getVirtualAdapter(self, frameName, viosName):
        log.info("Inside getVirtualAdapter method start")
        viosCmd = self.viosHandler.getVirtAdapter()
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        adapterList = []
        if response['code'] == 0:
            for line in response['stdout']:
                adpt = line.split(' ', 1)
                adapterList.append(adpt[0])
        else:
            for line in response['stdout']:
                print(line)
        return adapterList

    # Get Port VLAN ID
    def getPortVlanID(self, frameName, viosName, vEth):
        log.info("Inside getPortVlanID method start")
        viosCmd = self.viosHandler.getPortVlanID(vEth)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        vLanId = False
        if response['code'] == 0:
            for line in response['stdout']:
                vlans = line.split(':')
                vLanId = vlans[1].replace("\n", "").strip()
        else:
            for line in response['stdout']:
                print(line)
        return vLanId

    # Get Physical Adapters
    def getPhysicalAdapter(self, frameName, viosName):
        log.info("Inside getPhysicalAdapter method start")
        viosCmd = self.viosHandler.getPhysicalAdapter()
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        phyAdpt = False
        if response['code'] == 0:
            for line in response['stdout']:
                adpt = line.split(' ', 1)
                phyAdpt = adpt[0].strip()
        else:
            for line in response['stdout']:
                print(line)
        return phyAdpt

    # Create SEA
    def createSEA(self, frameName, viosName, phyEth, vEth, vLAN):
        log.info("Inside createSEA method start")
        viosCmd = self.viosHandler.createSEA(phyEth, vEth, vLAN)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        try:
            GOLD_BUILD = frameName+viosName
            GOLD_BUILD = GOLD_BUILD.replace("-", "").replace("_", "")
            self.clearOlderResOfInstallios(GOLD_BUILD)
        except Exception as e:
            print("Unable to remove installios resources")
            log.error(e)
        return response

    def get_vios_hostname(self, frameName, viosName):
        log.info("Inside get_vios_hostname method start")
        viosCmd = self.viosHandler.get_hostname()
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        hostname = False
        if response['code'] == 0:
            for line in response['stdout']:
                hostname = line.strip()
                log.info("Hostname found as : "+str(hostname))
        else:
            for line in response['stdout']:
                log.error("Unable to find hostname")
                log.error(line)
        return hostname

    def filter_mgmt_switch_adapter(
            self, frameName, viosName, interface, vlan_id):
        log.info("Inside filter_management_switch_adapter method start")
        viosCmd = self.viosHandler.get_mgmt_switch_interface(interface)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        if response['code'] == 0:
            for line in response['stdout']:
                vlans = line.split(':')
                vLanId = vlans[1].replace("\n", "").strip()
                if vLanId == vlan_id:
                    return True
                else:
                    return False
        else:
            for line in response['stdout']:
                print(line)
            return False

    def get_empty_interface_vios(self, frameName, viosName, vlan_id):
        log.info("Inside get_empty_interface method start")
        viosCmd = self.viosHandler.get_empty_interface()
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        interface = False
        if response['code'] == 0:
            for line in response['stdout']:
                adpt = line.split(' ', 1)
                interface = adpt[0].strip()
                log.info("Interface found as : "+str(interface))
                res = self.filter_mgmt_switch_adapter(
                    frameName, viosName, interface, vlan_id)
                if res:
                    log.info("Interface found with mgmt switch vlan id: "+str(
                        interface))
                    return interface
        else:
            for line in response['stdout']:
                log.error("Unable to find empty interface")
                log.error(line)
        return interface

    def add_network_on_vios(
            self, frameName, viosName, hostname, ip_add, netmask, interface):
        log.info("Inside add_network_on_vios method start")
        viosCmd = self.viosHandler.add_network(
            hostname, ip_add, netmask, interface)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        if response['code'] == 0:
            log.info("Private network added successfully :"+str(viosName))
            return True
        else:
            for line in response['stdout']:
                log.error("Unable to add private network for "+str(viosName))
                log.error(line)
        return False

    # Set SRIOV Adapters mode
    def setSRIOVMode(self, frameName, slot_id, sriov_mode):
        log.info("Inside setSRIOVSharedMode method start")
        mode = "r"  # default to dedicated mode
        if sriov_mode == "sriov":
            mode = "a"
        command = 'chhwres -r sriov --rsubtype adapter -m '+str(
            frameName)+' -o '+mode+' -a "slot_id='+str(slot_id)+'"'
        response = self.executeCommand(command)
        return response

    # Set shared processor pool attr for VIOS and Novalink
    # Max Proc : 4
    # Reserved Proc : 0
    def changeSharedProcPoolAttr(
            self, frameName, poolId, poolName, maxProc=4, resvrdProc=0):
        log.info("Inside changeSharedProcPoolAttr method start")
        command = 'chhwres --force -r procpool -m '+str(
            frameName)+' -o s --poolid '+str(poolId)+' -a "new_name='+str(
            poolName)+',max_pool_proc_units='+str(
            maxProc)+',reserved_pool_proc_units='+str(resvrdProc)+'"'
        response = self.executeCommand(command)
        return response

    # Create management vswitch for VIOS and Novalink
    def create_mgmt_vswitch(self, frameName, switchname):
        log.info("Inside create_mgmt_vswitch method start")
        command = 'chhwres -r virtualio --rsubtype vswitch -m '+str(
            frameName)+' -o a --vswitch '+str(switchname)+' --mode=Vepa'
        response = self.executeCommand(command)
        return response

    # Set HMC as PowerVM management master
    def coMgmtSetMaster(self, frameName):
        log.info("Inside coMgmtSetMaster method start")
        command = 'chcomgmt --force -o setmaster -t norm -m '+frameName
        response = self.executeCommand(command)
        return response

    # Release HMC from PowerVM management master
    def coMgmtReleaseMaster(self, frameName):
        log.info("Inside coMgmtReleaseMaster method start")
        command = 'chcomgmt --force -o relmaster -m '+frameName
        response = self.executeCommand(command)
        return response

    # powervm_mgmt_capable : 0 - do not allow for PowerVM management
    # powervm_mgmt_capable : 1 - Enable partition for PowerVM management
    def powerVMcapableManagement(self, frameName, lparId, action):
        log.info("Inside powerVMcapableManagement method start")
        command = 'chsyscfg --force -m '+frameName+' -r lpar -i lpar_id='+str(
            lparId)+',powervm_mgmt_capable='+str(action)
        response = self.executeCommand(command)
        return response

    # Create Logical Volume
    def createLV(self, info):
        log.info("Inside createLV method start")
        diskName = info['diskName']
        diskSize = info['diskSize']
        frameName = info['frameName']
        viosName = info['viosName']
        vg = info['diskVG']
        viosCmd = self.viosHandler.createLV(diskName, diskSize, vg)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        return response

    # Get vHost name using lpar slot number
    def getvHost(self, info):
        log.info("Inside getvHost method start")
        frameName = info['frameName']
        viosName = info['viosName']
        lparSlotNum = info['lparSlotNum']
        command = 'lshwres -r virtualio --rsubtype slot -m '+str(
            frameName)+' --level slot --filter "lpar_names="'+str(
            viosName)+' | grep -w slot_num='+str(lparSlotNum)
        response = self.executeCommand(command)
        data = {}
        for line in response['stdout']:
            res = self.formatHmcOutput(line)
            data = self.parseCommaSeparatedData(res)
        drc_name = data.get("drc_name")
        viosCommand = self.viosHandler.getvHostUsingDrc(drc_name)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCommand
        response = self.executeCommand(command)
        return response

    # Map logical volume(disk) to vHost
    def mapDiskTovHost(self, info, vHost):
        log.info("Inside mapDiskTovHost method start")
        diskName = info['diskName']
        frameName = info['frameName']
        viosName = info['viosName']
        viosCmd = self.viosHandler.diskvHostMapping(diskName, vHost)
        command = VIOS_CMD+' -m '+frameName+' -p '+viosName+' -c '+viosCmd
        response = self.executeCommand(command)
        return response

    def getLocCodeNovalink(self, info):
        frameName = info['frameName']
        lparName = info['lparName']
        profileName = info['profileName']
        serverIp = info['serverIp']
        lparIp = info['lparIp']
        subnetMask = info['subnetMask']
        gatewayIp = info['gatewayIp']
        # Parameters details
        # -i : Force immediate shutdown
        # -A : Return all adapters of the given type
        # -M : Discovery ethernet adapter mac address and location code
        # -f : Force close virtual terminal session
        # -T : Enable or disble spanning tree
        # -D : Perform ping test, use adapter that successfully pings server
        # -K : Subnetmask IP address
        # -G : Gateway IP address
        # -C : Client IP address (VIOS/VM IP)
        # -S : Server IP (BootP)
        # -d : Network adapter duplex
        # -s : Network adapter speed
        # -t : Specifies network type ent
        # -n : Do not boot partition

        command = "lpar_netboot -i -A -M -f -T off -D -K "+str(
            subnetMask)+" -G "+gatewayIp+" -C "+lparIp+" -S "+str(
            serverIp)+" -d auto -s auto -t ent -n "+lparName+" "+str(
            frameName)
        try:
            log.info(command)
            stdin, stdout, stderr = self.sshClient.exec_command(command)
            for x in stdout:
                print(x)
                log.info(str(x))
                PING_PASS = CONF.get(
                    'novalink_profile', 'LPAR_PING_SUCCESS_STR')
                PING_FAIL = CONF.get('novalink_profile', 'LPAR_PING_FAIL_STR')
                if PING_PASS in x and PING_FAIL not in x:
                    print("Proceeding for Novalink Installation")
                    return self.formatResponse(0, x, stderr)
            return self.formatResponse(1, stdout, stderr)
        except Exception as e:
            print(str(e))
            return self.formatResponse(3, str(e), str(e))

    def installNovalink(self, info):
        log.info("Inside installNovalink start")
        frameName = info['frameName']
        lparName = info['lparName']
        profileName = info['profileName']
        serverIp = info['serverIp']
        lparIp = info['lparIp']
        subnetMask = info['subnetMask']
        gatewayIp = info['gatewayIp']
        loc_code = info['loc_code']
        time.sleep(20)
        self.checkLparShutDownState(frameName, lparName)
        # Parameters details
        # -i : Force immediate shutdown
        # -A : Return all adapters of the given type
        # -M : Discovery ethernet adapter mac address and location code
        # -f : Force close virtual terminal session
        # -T : Enable or disble spanning tree
        # -D : Perform ping test, use adapter that successfully pings server
        # -K : Subnetmask IP address
        # -G : Gateway IP address
        # -C : Client IP address (VIOS/VM IP)
        # -S : Server IP (BootP)
        # -d : Network adapter duplex
        # -s : Network adapter speed
        # -t : Specifies network type ent
        # -n : Do not boot partition
        cmd = "lpar_netboot -i -f -T off -t ent -s auto -d auto -l "+str(
            loc_code)+" -K "+str(subnetMask)+" -G "+gatewayIp+" -C "+str(
            lparIp)+" -S "+serverIp+" "+str(
            lparName)+" "+frameName
        log.info(cmd)
        try:
            stdin, stdout, stderr = self.sshClient.exec_command(cmd)
            for x in stdout:
                log.info(str(x))
                print(x)
                if 'unsuccessful' in x:
                    print("Ping failed...")
                    return self.formatResponse(1, x, stderr)
                if 'Finished' in x:
                    return self.formatResponse(0, x, stderr)
            print("Error Occurred")
            self.formatResponse(2, stdout, stderr)
            print("Re-trying Novalink Installation, Please wait ...")
            log.info("Re-trying Novalink Installation")
            time.sleep(30)
            log.info(cmd)
            stdin, stdout, stderr = self.sshClient.exec_command(cmd)
            for x in stdout:
                log.info(str(x))
                print(x)
                if 'unsuccessful' in x:
                    print("Ping failed...")
                    return self.formatResponse(1, x, stderr)
                if 'Finished' in x:
                    return self.formatResponse(0, x, stderr)
            print("Error Occurred")
            return self.formatResponse(2, stdout, stderr)
        except Exception as e:
            print(str(e))
            return self.formatResponse(3, str(e), str(e))

    # Configure Novalink
    def configureNovalinkVm(self, hostName):
        log.info("Setting sys request-master")
        command_1 = "pvmctl sys request-master"
        res1 = self.executeCommand(command_1)
        code1 = res1['code']
        log.info(str(code1))
        if code1 == 0:
            print(MSGCATALOG.NOVALINK_SET_MASTER)
            log.info(MSGCATALOG.NOVALINK_SET_MASTER)
        else:
            print(MSGCATALOG.NOVALINK_PVM_PACKAGE_ERROR)
            log.info(MSGCATALOG.NOVALINK_PVM_PACKAGE_ERROR)

        command_2 = "cat /etc/os-release | grep -w NAME="
        res2 = self.executeCommand(command_2)
        os_name = ""
        if res2['code'] == 0:
            for info in res2['stdout']:
                os_name = info.lower()
        log.info("OS Name found as : "+str(os_name))
        return code1, os_name

    def disable_firewall(self, hostname):
        log.info("inside disable_firewall method")
        command_2 = "hostnamectl set-hostname "+str(hostname)
        res2 = self.executeCommand(command_2)
        code2 = res2['code']
        log.info(str(code2))
        for info in res2['stdout']:
            log.info(str(info))
        log.info("Firewall service stop")
        command_3 = "systemctl stop firewalld"
        res3 = self.executeCommand(command_3)
        code3 = res3['code']
        log.info(str(code3))
        for info in res3['stdout']:
            log.info(str(info))
        return code3

    # Get pvmctl version
    def get_pvmctl_version(self,):
        command = "pvmctl --version"
        response = self.executeCommand(command)
        return response
